#ifndef __PUBSUB_H__
#define __PUBSUB_H__

#include <open62541/server.h>
#include <open62541/plugin/log_stdout.h>

#include <quickjs/quickjs.h>

#define DLL_PUBLIC __attribute__ ((visibility ("default")))

#define countof(x) (sizeof(x) / sizeof((x)[0]))

#define DATA_SET_WRITER_ID 1
#define WRITER_GROUP_ID 1

#define MAX_MESSAGE_SIZE 1024

typedef struct {
    UA_UInt16 id;
    UA_Double latitude;
    UA_UInt32 latitudeId;
    UA_Double longitude;
    UA_UInt32 longitudeId;
    UA_Float altitudeAbs;
    UA_UInt32 altitudeAbsId;
    UA_Float altitudeRel;
    UA_UInt32 altitudeRelId;
    char message[MAX_MESSAGE_SIZE];
    UA_UInt32 messageId;
} JSDroneData;

typedef struct {
    char *name;
    char *description;
    void * UA_RESTRICT value;
    int type;
    UA_Byte builtInType;
    union {
        UA_Double   (*getDouble)(void);
        UA_Float    (*getFloat)(void);
        UA_String   (*getString)(void);
    } getter;

} VariableData;

int subscribeOnly(UA_String *transportProfile,
                  UA_NetworkAddressUrlDataType *networkAddressUrl,
                  VariableData *variableArray, size_t nbVariable,
                  UA_UInt32 id, UA_UInt32 nbReader, UA_Duration interval,
                  void (*init_node_id)(UA_UInt32 id, UA_UInt32 nb, UA_UInt32 magic),
                  UA_UInt16 (*get_reader_id)(UA_UInt32 nb),
                  void (*update)(UA_UInt32 id, const UA_DataValue*),
                  UA_Boolean *running);

int runPubsub(UA_String *transportProfile,
              UA_NetworkAddressUrlDataType *networkAddressUrl,
              VariableData *variableArray, size_t nbVariable, 
              UA_UInt32 id, UA_UInt32 nbReader, UA_Duration interval,
              void (*init_node_id)(UA_UInt32 id, UA_UInt32 nb, UA_UInt32 magic),
              UA_UInt16 (*get_reader_id)(UA_UInt32 nb),
              VariableData (*get_value)(UA_String identifier),
              void (*update)(UA_UInt32 id, const UA_DataValue*),
              UA_Boolean *running);

UA_String get_message(void);

UA_UInt16 get_drone_id(UA_UInt32 nb);

void init_node_id(UA_UInt32 id, UA_UInt32 nb, UA_UInt32 magic);

void pubsub_update_coordinates(UA_UInt32 id, const UA_DataValue *var);

void pubsub_print_coordinates(UA_UInt32 id, const UA_DataValue *var);

VariableData pubsub_get_value(UA_String identifier);

DLL_PUBLIC JSModuleDef *js_init_module(JSContext *ctx, const char *module_name);

#endif /* __PUBSUB_H__ */
